@@ -4,51 +4,55 @@
 # 3. "A First Course in the Finite Element Method, 4th Edition", Daryl L. Logan
 # 4. "Finite Element Analysis Fundamentals", Richard H. Gallagher
 
+from __future__ import annotations  # Allows more recent type hints features
+from typing import TYPE_CHECKING, Literal
+
+from math import sin, cos
 import numpy as np
 from numpy import add
 from numpy.linalg import inv, det, norm
-from math import sin, cos
+import warnings
+
+if TYPE_CHECKING:
+    from typing import List, Tuple, Optional
+    from numpy import float64
+    from numpy.typing import NDArray
+    from Pynite.FEModel3D import FEModel3D
+    from Pynite.Node3D import Node3D
 
 class Quad3D():
     """
-    An isoparametric general quadrilateral element, formulated by superimposing an isoparametric
-    DKMQ bending element with an isoparametric plane stress element. Drilling stability is
-    provided by adding a weak rotational spring stiffness at each node. Isotropic behavior is the
-    default, but orthotropic in-plane behavior can be modeled by specifying stiffness modification
-    factors for the element's local x and y axes. Orthotropic behavior is only available for
-    rectangular plates.
+    An isoparametric general quadrilateral element, formulated by superimposing an isoparametric DKMQ bending element with an isoparametric plane stress element. Drilling stability is provided by adding a weak rotational spring stiffness at each node. Isotropic behavior is the default, but orthotropic in-plane behavior can be modeled by specifying stiffness modification factors for the element's local x and y axes.
 
-    This element performs well for thick and thin plates, and for skewed plates. Element center
-    stresses and corner FORCES converge rapidly; however, corner STRESSES are more representative
-    of center stresses. Minor errors are introduced into the solution due to the drilling
-    approximation. Orthotropic behavior is limited to acting along the plate's local axes.
+    This element performs well for thick and thin plates, and for skewed plates. Minor errors are introduced into the solution due to the drilling approximation. Orthotropic behavior is limited to acting along the plate's local axes.
     """
 
-    def __init__(self, name, i_node, j_node, m_node, n_node, t, material_name, model, kx_mod=1.0,
-                 ky_mod=1.0):
+    def __init__(self, name: str, i_node: Node3D, j_node: Node3D, m_node: Node3D, n_node: Node3D, 
+                 t: float, material_name: str, model: FEModel3D, kx_mod: float = 1.0,
+                 ky_mod: float = 1.0):
 
-        self.name = name
-        self.ID = None
-        self.type = 'Quad'
+        self.name: str = name
+        self.ID: Optional[int] = None
+        self.type: str = 'Quad'
 
-        self.i_node = i_node
-        self.j_node = j_node
-        self.m_node = m_node
-        self.n_node = n_node
+        self.i_node: Node3D = i_node
+        self.j_node: Node3D = j_node
+        self.m_node: Node3D = m_node
+        self.n_node: Node3D = n_node
 
-        self.t = t
-        self.kx_mod = kx_mod
-        self.ky_mod = ky_mod
+        self.t: float = t
+        self.kx_mod: float = kx_mod
+        self.ky_mod: float = ky_mod
+
+        self.pressures: List[Tuple[float, str]] = []  # A list of surface pressures [pressure, case='Case 1']
 
-        self.pressures = []  # A list of surface pressures [pressure, case='Case 1']
-    
         # Quads need a link to the model they belong to
-        self.model = model
+        self.model: FEModel3D = model
 
         # Get material properties for the plate from the model
         try:
-            self.E = self.model.materials[material_name].E
-            self.nu = self.model.materials[material_name].nu
+            self.E: float = self.model.materials[material_name].E
+            self.nu: float = self.model.materials[material_name].nu
         except:
             raise KeyError('Please define the material ' + str(material_name) + ' before assigning it to plates.')
 
@@ -96,7 +100,7 @@ class Quad3D():
     #     self.y2 = np.dot(vector_02, y_axis)
     #     self.y3 = np.dot(vector_03, y_axis)
     #     self.y4 = np.dot(vector_04, y_axis)
-    
+
     def _local_coords(self):
         """
         Calculates or recalculates and stores the local (x, y) coordinates for each node of the quadrilateral.
@@ -124,16 +128,16 @@ class Quad3D():
         y_axis = y_axis/norm(y_axis)
 
         # Calculate the local (x, y) coordinates for each node
-        self.x1 = 0
-        self.x2 = np.dot(vector_12, x_axis)
-        self.x3 = np.dot(vector_13, x_axis)
-        self.x4 = np.dot(vector_14, x_axis)
-        self.y1 = 0
-        self.y2 = np.dot(vector_12, y_axis)
-        self.y3 = np.dot(vector_13, y_axis)
-        self.y4 = np.dot(vector_14, y_axis)
+        self.x1: float = 0.0
+        self.x2: float = np.dot(vector_12, x_axis)
+        self.x3: float = np.dot(vector_13, x_axis)
+        self.x4: float = np.dot(vector_14, x_axis)
+        self.y1: float = 0.0
+        self.y2: float = np.dot(vector_12, y_axis)
+        self.y3: float = np.dot(vector_13, y_axis)
+        self.y4: float = np.dot(vector_14, y_axis)
 
-    def L_k(self, k):
+    def L_k(self, k: Literal[5, 6, 7, 8]) -> float:
         
         # Figures 3 and 5
         if k == 5:
@@ -147,7 +151,7 @@ class Quad3D():
         else:
             raise Exception('Invalid value for k. k must be 5, 6, 7, or 8.')
     
-    def dir_cos(self, k):
+    def dir_cos(self, k: Literal[5, 6, 7, 8]) -> Tuple[float, float]:
 
         L_k = self.L_k(k)
 
@@ -169,14 +173,14 @@ class Quad3D():
 
         return C, S
 
-    def phi_k(self, k):
+    def phi_k(self, k: Literal[5, 6, 7, 8]) -> float:
 
         kappa = 5/6
 
         # Equation 74
         return 2/(kappa*(1-self.nu))*(self.t/self.L_k(k))**2
 
-    def N_i(self, i, xi, eta):
+    def N_i(self, i: Literal[1, 2, 3, 4], xi: float, eta: float) -> float:
         """
         Returns the interpolation function for any given coordinate in the natural (xi, eta) coordinate system
         """
@@ -192,7 +196,7 @@ class Quad3D():
         else:
             raise Exception('Unable to calculate interpolation function. Invalid value specifed for i.')
     
-    def P_k(self, k, xi, eta):
+    def P_k(self, k: Literal[5, 6, 7, 8], xi: float, eta: float) -> float:
 
         if k == 5:
             return 1/2*(1 - xi**2)*(1 - eta)
@@ -205,8 +209,9 @@ class Quad3D():
         else:
             raise Exception('Unable to calculate shape function. Invalid value specified for k.')
     
-    def Co(self, xi, eta):
-        """This alternate calculation of the Jacobian matrix follows "The development of DKMQ plate bending element for thick to thin shell analysis based on the Naghdi/Reissner/Mindlin shell theory" by Katili, Batoz, Maknun and Hamdouni (2015). In the reference "C^o" is used instead of "J" to refer to the Jacobian. This method does not seem to produce correct results, but will be kept for future reference. It is helpful for understanding this plate element and may prove a useful simplification to the code base if implemented correctly someday.
+    def Co(self, xi: float, eta: float) -> NDArray[float64]:
+        """
+        This alternate calculation of the Jacobian matrix follows "The development of DKMQ plate bending element for thick to thin shell analysis based on the Naghdi/Reissner/Mindlin shell theory" by Katili, Batoz, Maknun and Hamdouni (2015). In the reference "C^o" is used instead of "J" to refer to the Jacobian. This method does not seem to produce incorrect results, but will be kept for future reference. It is helpful for understanding this plate element and may prove a useful simplification to the code base if implemented correctly someday.
         """
 
         # Nodal global coordinates (i=1, j=2, m=3, n=4)
@@ -236,6 +241,7 @@ class Quad3D():
         i = np.array([[1.0, 0.0, 0.0]])
         k = np.array([[0.0, 0.0, 1.0]])
 
+        # Equation (13)
         if np.array_equal(n, k) or np.array_equal(n, -k):
             t_1 = i
         else:
@@ -243,16 +249,19 @@ class Quad3D():
 
         t_2 = np.cross(n, t_1)
 
+        # Equation (7)
         a_11 = np.dot(a_1, a_1.T)[0, 0]
         a_12 = np.dot(a_1, a_2.T)[0, 0]
         a_21 = np.dot(a_2, a_1.T)[0, 0]
         a_22 = np.dot(a_2, a_2.T)[0, 0]
 
+        # Matrix tensor of the middle surface
         a = np.array([[a_11, a_12],
                       [a_21, a_22]])
         
         a_det = np.linalg.det(a)
 
+        # Contravariant vectors
         a1 = 1/a_det*(a_22*a_1 - a_12*a_2)
         a2 = 1/a_det*(-a_21*a_1 + a_11*a_2)
 
@@ -261,11 +270,11 @@ class Quad3D():
         
         return Co
 
-    def J(self, xi, eta):
+    def J(self, xi: float, eta: float) -> NDArray[float64]:
         """
         Returns the Jacobian matrix for the element
         """
-        
+
         # Get the local coordinates for the element
         x1, y1, x2, y2, x3, y3, x4, y4 = self.x1, self.y1, self.x2, self.y2, self.x3, self.y3, self.x4, self.y4
 
@@ -273,13 +282,13 @@ class Quad3D():
         return 1/4*np.array([[x1*(eta - 1) - x2*(eta - 1) + x3*(eta + 1) - x4*(eta + 1), y1*(eta - 1) - y2*(eta - 1) + y3*(eta + 1) - y4*(eta + 1)],
                              [x1*(xi - 1)  - x2*(xi + 1)  + x3*(xi + 1)  - x4*(xi - 1),  y1*(xi - 1)  - y2*(xi + 1)  + y3*(xi + 1)  - y4*(xi - 1)]])
 
-    def N_gamma(self, xi, eta):
+    def N_gamma(self, xi: float, eta: float) -> NDArray[float64]:
 
         # Equation 44
         return np.array([[1/2*(1 - eta),       0,      1/2*(1 + eta),       0     ],
                          [     0,        1/2*(1 + xi),       0,       1/2*(1 - xi)]])
 
-    def A_gamma(self):
+    def A_gamma(self) -> NDArray[float64]:
 
         L5 = self.L_k(5)
         L6 = self.L_k(6)
@@ -292,7 +301,7 @@ class Quad3D():
                          [  0,    0,  -L7/2,    0 ],
                          [  0,    0,     0,  -L8/2]])
 
-    def A_u(self):
+    def A_u(self) -> NDArray[float64]:
 
         # Calculate the length of each side of the quad
         L5 = self.L_k(5)
@@ -312,7 +321,7 @@ class Quad3D():
                              [  0,    0,  0,   0,    0,  0, -2/L7, C7, S7,  2/L7, C7, S7],
                              [ 2/L8, C8, S8,   0,    0,  0,   0,    0,  0, -2/L8, C8, S8]])
 
-    def A_Delta_inv_DKMQ(self):
+    def A_Delta_inv_DKMQ(self) -> NDArray[float64]:
 
         phi5 = self.phi_k(5)
         phi6 = self.phi_k(6)
@@ -324,7 +333,7 @@ class Quad3D():
                               [   0,           0,      1/(1+phi7),     0     ],
                               [   0,           0,          0,      1/(1+phi8)]])
 
-    def A_phi_Delta(self):
+    def A_phi_Delta(self) -> NDArray[float64]:
 
         phi5 = self.phi_k(5)
         phi6 = self.phi_k(6)
@@ -336,7 +345,7 @@ class Quad3D():
                          [      0,             0,       phi7/(1+phi7),       0      ],
                          [      0,             0,             0,       phi8/(1+phi8)]])
     
-    def B_b_beta(self, xi, eta):
+    def B_b_beta(self, xi: float, eta: float) -> NDArray[float64]:
 
         # Get the inverse of the Jacobian matrix
         J_inv = inv(self.J(xi, eta))
@@ -370,7 +379,7 @@ class Quad3D():
                          [0,  0,  N1y, 0,  0,  N2y, 0,  0,  N3y, 0,  0,  N4y],
                          [0, N1y, N1x, 0, N2y, N2x, 0, N3y, N3x, 0, N4y, N4x]])
     
-    def B_b_Delta_beta(self, xi, eta):
+    def B_b_Delta_beta(self, xi: float, eta: float) -> NDArray[float64]:
 
         # Get the inverse of the Jacobian matrix
         J_inv = inv(self.J(xi, eta))
@@ -409,7 +418,7 @@ class Quad3D():
                          [    P5y*S5,          P6y*S6,          P7y*S7,          P8y*S8,    ],
                          [P5y*C5 + P5x*S5, P6y*C6 + P6x*S6, P7y*C7 + P7x*S7, P8y*C8 + P8x*S8]])
     
-    def B_b(self, xi, eta):
+    def B_b(self, xi: float, eta: float) -> NDArray[float64]:
         """
         Returns the [B_b] matrix for bending
         """
@@ -417,7 +426,7 @@ class Quad3D():
         # Return the [B] matrix for bending
         return add(self.B_b_beta(xi, eta), self.B_b_Delta_beta(xi, eta) @ self.A_Delta_inv_DKMQ() @ self.A_u())
 
-    def B_s(self, xi, eta):
+    def B_s(self, xi: float, eta: float) -> NDArray[float64]:
         """
         Returns the [B_s] matrix for shear
         """
@@ -425,7 +434,17 @@ class Quad3D():
         # Return the [B] matrix for shear
         return inv(self.J(xi, eta)) @ self.N_gamma(xi, eta) @ self.A_gamma() @ self.A_phi_Delta() @ self.A_u()
 
-    def B_m(self, xi, eta):
+    def B_s_gamma(self, xi:float , eta: float) -> None:
+        """Returns the [B_s_gamma] matrix for shear (Equation 39 in Reference 1)
+
+        :param xi: _description_
+        :type xi: _type_
+        :param eta: _description_
+        :type eta: _type_
+        """
+        raise NotImplementedError('This function is not implemented yet. It is not needed for the current implementation of the Quad3D element.')
+
+    def B_m(self, xi: float, eta: float) -> NDArray[float64]:
 
         # Differentiate the interpolation functions
         # Row 1 = interpolation functions differentiated with respect to x
@@ -442,7 +461,7 @@ class Quad3D():
 
         return B_m
 
-    def Hb(self):
+    def Hb(self) -> NDArray[float64]:
         '''
         Returns the stress-strain matrix for plate bending.
         '''
@@ -458,7 +477,7 @@ class Quad3D():
         
         return Hb
 
-    def Hs(self):
+    def Hs(self) -> NDArray[float64]:
         '''
         Returns the stress-strain matrix for shear.
         '''
@@ -473,7 +492,7 @@ class Quad3D():
 
         return Hs
 
-    def Cm(self):
+    def Cm(self) -> NDArray[float64]:
         """
         Returns the stress-strain matrix for an isotropic or orthotropic plane stress element
         """
@@ -495,12 +514,12 @@ class Quad3D():
         Cm = 1/(1 - nu_xy*nu_yx)*np.array([[   Ex,    nu_yx*Ex,           0         ],
                                            [nu_xy*Ey,    Ey,              0         ],
                                            [    0,        0,     (1 - nu_xy*nu_yx)*G]])
-        
+
         return Cm
 
-    def k_b(self):
+    def k_b(self) -> NDArray[float64]:
         '''
-        Returns the local stiffness matrix for bending stresses
+        Returns the local stiffness matrix for bending and shear stresses
         '''
 
         Hb = self.Hb()
@@ -510,10 +529,10 @@ class Quad3D():
         gp = 1/3**0.5
 
         # Get the determinant of the Jacobian matrix for each gauss point. Doing this now will save us from doing it twice below.
-        J1 = det(self.J(-gp, -gp))
-        J2 = det(self.J( gp, -gp))
-        J3 = det(self.J( gp,  gp))
-        J4 = det(self.J(-gp,  gp))
+        det_J1 = det(self.J(-gp, -gp))
+        det_J2 = det(self.J( gp, -gp))
+        det_J3 = det(self.J( gp,  gp))
+        det_J4 = det(self.J(-gp,  gp))
 
         # Get the bending [B_b] matrices for each gauss point
         B1 = self.B_b(-gp, -gp)
@@ -523,10 +542,10 @@ class Quad3D():
 
         # Create the stiffness matrix with bending stiffness terms
         # See 2, Equation 5.94
-        k = ((B1.T @ Hb @ B1)*J1 +
-             (B2.T @ Hb @ B2)*J2 +
-             (B3.T @ Hb @ B3)*J3 +
-             (B4.T @ Hb @ B4)*J4)
+        k = ((B1.T @ Hb @ B1)*det_J1 +
+             (B2.T @ Hb @ B2)*det_J2 +
+             (B3.T @ Hb @ B3)*det_J3 +
+             (B4.T @ Hb @ B4)*det_J4)
 
         # Get the shear [B_s] matrices for each gauss point
         B1 = self.B_s(-gp, -gp)
@@ -535,10 +554,10 @@ class Quad3D():
         B4 = self.B_s(-gp,  gp)
 
         # Add shear stiffness terms to the stiffness matrix
-        k += ((B1.T @ Hs @ B1)*J1 +
-              (B2.T @ Hs @ B2)*J2 +
-              (B3.T @ Hs @ B3)*J3 +
-              (B4.T @ Hs @ B4)*J4)
+        k += ((B1.T @ Hs @ B1)*det_J1 +
+              (B2.T @ Hs @ B2)*det_J2 +
+              (B3.T @ Hs @ B3)*det_J3 +
+              (B4.T @ Hs @ B4)*det_J4)
         
         # Following Bathe's recommendation for the drilling degree of freedom
         # from Example 4.19 in "Finite Element Procedures, 2nd Ed.", calculate
@@ -602,13 +621,15 @@ class Quad3D():
         k_exp[[4, 10, 16, 22], :] *= -1
         k_exp[:, [4, 10, 16, 22]] *= -1
 
-        # The way the DKMQ element was derived, the local x and y axes of the element are swapped from Pynite's global definitions of x and y. Swap them to match Pynite.
+        # The way the DKMQ element was derived, the positions relating to x
+        # and y in the element's stiffness matrix are swapped from Pynite's.
+        # Swap them to match Pynite.
         k_exp[[3, 4, 9, 10, 15, 16, 21, 22], :] = k_exp[[4, 3, 10, 9, 16, 15, 22, 21], :]
         k_exp[:, [3, 4, 9, 10, 15, 16, 21, 22]] = k_exp[:, [4, 3, 10, 9, 16, 15, 22, 21]]
 
         return k_exp
 
-    def k_m(self):
+    def k_m(self) -> NDArray[float64]:
         '''
         Returns the local stiffness matrix for membrane (in-plane) stresses.
 
@@ -628,11 +649,20 @@ class Quad3D():
         B3 = self.B_m( gp,  gp)
         B4 = self.B_m(-gp,  gp)
 
+        # Calculate the determinant of the Jacobian matrix at each gauss point
+        det_J1 = det(self.J(-gp, -gp))
+        det_J2 = det(self.J(gp, -gp))
+        det_J3 = det(self.J(gp, gp))
+        det_J4 = det(self.J(-gp, gp))
+
+        if det_J1 <= 0 or det_J2 <= 0 or det_J3 <= 0 or det_J4 <= 0:
+            warnings.warn(f'The Jacobian matrix for quad element {self.name} is less than or equal to zero, indicating the element is invalid or badly distorted.')
+
         # See reference 2 at the bottom of page 353, and reference 2 page 466
-        k = t*((B1.T @ Cm @ B1)*det(self.J(-gp, -gp)) +
-               (B2.T @ Cm @ B2)*det(self.J( gp, -gp)) +
-               (B3.T @ Cm @ B3)*det(self.J( gp,  gp)) +
-               (B4.T @ Cm @ B4)*det(self.J(-gp,  gp)))
+        k = t*((B1.T @ Cm @ B1)*det_J1 +
+               (B2.T @ Cm @ B2)*det_J2 +
+               (B3.T @ Cm @ B3)*det_J3 +
+               (B4.T @ Cm @ B4)*det_J4)
         
         k_exp = np.zeros((24, 24))
 
@@ -666,7 +696,7 @@ class Quad3D():
         
         return k_exp
 
-    def k(self):
+    def k(self) -> NDArray[float64]:
         '''
         Returns the quad element's local stiffness matrix.
         '''
@@ -676,16 +706,16 @@ class Quad3D():
 
         # Sum the bending and membrane stiffness matrices
         return np.add(self.k_b(), self.k_m())
-   
-    def f(self, combo_name='Combo 1'):
+
+    def f(self, combo_name: str='Combo 1') -> NDArray[float64]:
         """
         Returns the quad element's local end force vector
         """
-        
+
         # Calculate and return the plate's local end force vector
         return np.add(self.k() @ self.d(combo_name), self.fer(combo_name))
 
-    def fer(self, combo_name='Combo 1'):
+    def fer(self, combo_name: str='Combo 1') -> NDArray[float64]:
         """
         Returns the quadrilateral's local fixed end reaction vector.
 
@@ -694,7 +724,10 @@ class Quad3D():
         combo_name : string
             The name of the load combination to get the consistent load vector for.
         """
-        
+
+        # Update the local coordinate system
+        self._local_coords()
+
         Hw = lambda xi, eta : 1/4*np.array([[(1 - xi)*(1 - eta), 0, 0, (1 + xi)*(1 - eta), 0, 0, (1 + xi)*(1 + eta), 0, 0, (1 - xi)*(1 + eta), 0, 0]])
 
         # Initialize the fixed end reaction vector
@@ -720,7 +753,7 @@ class Quad3D():
 
                     # Sum the pressures
                     p -= factor*pressure[0]
-        
+
         fer = (Hw(-gp, -gp).T*p*det(self.J(-gp, -gp))
              + Hw( gp, -gp).T*p*det(self.J( gp, -gp))
              + Hw( gp,  gp).T*p*det(self.J( gp,  gp))
@@ -732,7 +765,7 @@ class Quad3D():
         # Step through each term in the unexpanded vector
         # i = Unexpanded vector row
         for i in range(12):
-                
+
             # Find the corresponding term in the expanded vector
 
             # m = Expanded vector row
@@ -742,7 +775,7 @@ class Quad3D():
                 m = 2*i + 1
             if i in [2, 5, 8, 11]:  # indices associated with rotation about y
                 m = 2*i
-                
+
             # Ensure the index is an integer rather than a float
             m = round(m)
 
@@ -751,7 +784,7 @@ class Quad3D():
 
         return fer_exp
 
-    def d(self, combo_name='Combo 1'):
+    def d(self, combo_name='Combo 1') -> NDArray[float64]:
        """
        Returns the quad element's local displacement vector
        """
@@ -759,7 +792,7 @@ class Quad3D():
        # Calculate and return the local displacement vector
        return self.T() @ self.D(combo_name)
 
-    def F(self, combo_name='Combo 1'):
+    def F(self, combo_name: str='Combo 1') -> NDArray[float64]:
         """
         Returns the quad element's global force vector
 
@@ -768,11 +801,11 @@ class Quad3D():
         combo_name : string
             The load combination to get results for.
         """
-        
+
         # Calculate and return the global force vector
         return inv(self.T()) @ self.f(combo_name)
 
-    def D(self, combo_name='Combo 1'):
+    def D(self, combo_name:str='Combo 1') -> NDArray[float64]:
         '''
         Returns the quad element's global displacement vector for the given
         load combination.
@@ -815,11 +848,11 @@ class Quad3D():
         D[21, 0] = self.n_node.RX[combo_name]
         D[22, 0] = self.n_node.RY[combo_name]
         D[23, 0] = self.n_node.RZ[combo_name]
-        
+
         # Return the global displacement vector
         return D
 
-    def K(self):
+    def K(self) -> NDArray[float64]:
         '''
         Returns the quad element's global stiffness matrix
         '''
@@ -829,9 +862,9 @@ class Quad3D():
 
         # Calculate and return the stiffness matrix in global coordinates
         return inv(T) @ self.k() @ T
- 
+
     # Global fixed end reaction vector
-    def FER(self, combo_name='Combo 1'):
+    def FER(self, combo_name:str='Combo 1') -> NDArray[float64]:
         '''
         Returns the global fixed end reaction vector.
 
@@ -845,7 +878,7 @@ class Quad3D():
         # Calculate and return the fixed end reaction vector
         return inv(self.T()) @ self.fer(combo_name)
   
-    def T(self):
+    def T(self) -> NDArray[float64]:
         """
         Returns the coordinate transformation matrix for the quad element.
         """
@@ -978,7 +1011,7 @@ class Quad3D():
     #     # Return the transformation matrix.
     #     return T
     
-    def shear(self, xi=0, eta=0, local=True, combo_name='Combo 1'):
+    def shear(self, xi:float=0.0, eta:float=0.0, local:bool=True, combo_name:str='Combo 1') -> NDArray[float64]:
         """
         Returns the interal shears at any point in the quad element.
 
@@ -987,9 +1020,9 @@ class Quad3D():
 
         Parameters
         ----------
-        xi : number
+        xi : float
             The xi-coordinate. Default is 0.
-        eta : number
+        eta : float
             The eta-coordinate. Default is 0.
         
         Returns
@@ -1039,11 +1072,24 @@ class Quad3D():
             # Get the direction cosines for the plate's local coordinate system
             dir_cos = self.T()[:3, :3]
 
-            return np.matmul(dir_cos.T, np.array([Qx,
-                                                  Qy,
-                                                  [0]]))
-   
-    def moment(self, xi=0, eta=0, local=True, combo_name='Combo 1'):
+            # Transform the results to a global vector
+            Qx = float(Qx)
+            Qy = float(Qy)
+            Q_global = np.matmul(dir_cos.T, np.array([[Qx, 0,  0],
+                                                      [0,  Qy, 0],
+                                                      [0,  0,  0]]))
+
+            # Extract results acting along each global axis
+            Qx_global = Q_global[0, 0]
+            Qy_global = Q_global[1, 1]
+            Qz_global = Q_global[2, 1]
+
+            # Return the results as a 2D matrix
+            return np.array([[Qx_global],
+                             [Qy_global],
+                             [Qz_global]])
+
+    def moment(self, xi:float=0.0, eta:float=0.0, local:bool=True, combo_name:str='Combo 1') -> NDArray[float64]:
         """
         Returns the interal moments at any point in the quad element.
 
@@ -1052,11 +1098,11 @@ class Quad3D():
 
         Parameters
         ----------
-        xi : number
+        xi : float
             The xi-coordinate. Default is 0.
-        eta : number
+        eta : float
             The eta-coordinate. Default is 0.
-        
+
         Returns
         -------
         Internal moment per unit length of the quad element: [[Mx], [My], [Mxy]]
@@ -1064,7 +1110,7 @@ class Quad3D():
 
         # Get the plate's local displacement vector
         d = self.d(combo_name)
-                
+
         # Correct the sign convention for x-axis rotation - note that +x bending and +x rotation are opposite in the DKMQ derivation. Hence when correcting d we correct the x terms, but when correcting k we correct the y terms
         d[[3, 9, 15, 21], :] *= -1
 
@@ -1094,29 +1140,42 @@ class Quad3D():
         Mx = H[0]*m1[0] + H[1]*m2[0] + H[2]*m3[0] + H[3]*m4[0]
         My = H[0]*m1[1] + H[1]*m2[1] + H[2]*m3[1] + H[3]*m4[1]
         Mxy = H[0]*m1[2] + H[1]*m2[2] + H[2]*m3[2] + H[3]*m4[2]
-        
+
         if local:
 
             return np.array([Mx,
                              My,
                              Mxy])
-        
+
         else:
-            
+
             # Get the direction cosines for the plate's local coordinate system
             dir_cos = self.T()[:3, :3]
 
             # Convert the local results to global results
-            M_global = np.matmul(dir_cos, np.array([ -My,
-                                                      Mx,
-                                                      [0]]))
-            
-            M_global = M_global[[1, 0, 2], :]
+            Mx = float(Mx)
+            My = float(-My)
+            Mxy = float(Mxy)
+            M_local = np.array([
+                [Mx, Mxy, 0.0],
+                [Mxy, My, 0.0],
+                [0.0, 0.0, 0.0]
+            ])
 
-            return M_global
+            M_global_tensor = dir_cos @ M_local @ dir_cos.T
+
+            # Extract the results for each direction
+            Mx_global = M_global_tensor[0, 0]
+            My_global = M_global_tensor[1, 1]
+            Mxy_global = M_global_tensor[0, 1]
+
+            return np.array([[Mx_global],
+                             [My_global],
+                             [Mxy_global]])
+
+
+    def membrane(self, xi:float=0, eta: float=0, local:bool=True, combo_name:str='Combo 1') -> NDArray[float64]:
 
-    def membrane(self, xi=0, eta=0, local=True, combo_name='Combo 1'):
-        
         # Get the plate's local displacement vector. Slice out terms not related to membrane stresses.
         d = self.d(combo_name)[[0, 1, 6, 7, 12, 13, 18, 19], :]
 
@@ -1132,7 +1191,7 @@ class Quad3D():
 
         # Get the stress-strain matrix
         Cm = self.Cm()
-        
+
         # Calculate the internal stresses [Sx, Sy, Txy] at each gauss point
         s1 = np.matmul(Cm, np.matmul(self.B_m(-gp, -gp), d))
         s2 = np.matmul(Cm, np.matmul(self.B_m(gp, -gp), d))
@@ -1149,14 +1208,29 @@ class Quad3D():
             return np.array([Sx,
                              Sy,
                              Txy])
-        
+
         else:
 
             # Get the direction cosines for the plate's local coordinate system
             dir_cos = self.T()[:3, :3]
 
-            # Convert the plate membrane stresses to global coordinates
-            return np.matmul(dir_cos.T, np.array([Sx,
-                                                  Sy,
-                                                  [0]]))
+            # Convert the local results to global results
+            Sx = float(Sx)
+            Sy = float(Sy)
+            Txy = float(Txy)
+            S_local = np.array([
+                [Sx, Txy, 0.0],
+                [Txy, Sy, 0.0],
+                [0.0, 0.0, 0.0]
+            ])
 
+            S_global_tensor = dir_cos @ S_local @ dir_cos.T
+
+            # Extract the results for each direction
+            Sx_global = S_global_tensor[0, 0]
+            Sy_global = S_global_tensor[1, 1]
+            Sxy_global = S_global_tensor[0, 1]
+
+            return np.array([[Sx_global],
+                             [Sy_global],
+                             [Sxy_global]])
